
Game = Core.class(Sprite)

function Game:init()
  GAME = self
  
  local pack = TexturePack.new("images/gui.txt", "images/gui.png")
  self.guiPack = pack
  
  self.gravity = {x = 0, y = 32}

  self:generateWorld()

  self:addChild( Player.new() )
  
  self:addChild(Button.new(pack:getTextureRegion("jump.png"), pack:getTextureRegion("jump.png"), 
    {
      x         = WIDTH - 32,
      y         = HEIGHT - 32,
      alpha     = 0.25,
      onPressed = function(e) -- Player jumps immediately when button pressed
        local player = PLAYER
        if player.onGround then
          player.velocity.y = -768
          player:setState("jumping")
        end
      end,
    }))
  
  self:addChild( Button.new(pack:getTextureRegion("dPad.png"), pack:getTextureRegion("dPad.png"), 
    {
      x       = 72,
      y       = HEIGHT - 32,
      alpha   = 0.25,
      
      onPressed = function(e) -- Player changes direction and moves when dPad pressed or dragged
        local player = PLAYER
        if player.onGround then
          player:setState("running")
        end
        if e.touchX < 0 then
          player.clip:setScaleX(-1) -- Player facing and moving left
          player.velocity.x = -96
        else
          player.clip:setScaleX(1) -- Player facing and moving right
          player.velocity.x = 96
        end
      end,
      
      onDragged = function(e) -- Mirrored from above
        local player = PLAYER
        if player.onGround then
          player:setState("running")
        end
        if e.touchX < 0 then
          player.clip:setScaleX(-1)
          player.velocity.x = -96
        else
          player.clip:setScaleX(1)
          player.velocity.x = 96
        end
      end,
      
      onReleased = function(e) -- Player comes to a halt when dPad released
        local player = PLAYER
        if player.state ~= "dead" then
          player:setState("idle")
        end
        player.velocity.x = 0
      end,
    }))

  self:addEventListener(Event.ENTER_FRAME, self.update, self)
end

function Game:generateWorld() -- Make a test map
  local insert = table.insert
  
  local function rect(x, y, w, h, color) -- Draw a filled colored rectangle
    local rect = Shape.new()
    rect:setFillStyle(Shape.SOLID, color)  
    rect:beginPath()
    rect:moveTo(0, 0)
    rect:lineTo(w, 0)
    rect:lineTo(w, h)
    rect:lineTo(0, h)
    rect:closePath()
    rect:endPath()
    rect:setPosition(x, y)
    rect:setClip(0, 0, w, h)
    return rect
  end
  
  local worldObjects = Sprite.new()
  
  local groundObjects = {}
  local wallObjects = {}

  insert(groundObjects, rect( 32, 208, 160,  32, 0x008000) ) -- The ground
  insert(groundObjects, rect(256, 208, 160,  32, 0x008000) )
  
  insert(wallObjects,   rect(384, 144,  32,  64, 0x800000) ) -- Killer walls
  
  for i = 1, #groundObjects do
    worldObjects:addChild(groundObjects[i])
  end
  
  for i = 1, #wallObjects do
    worldObjects:addChild(wallObjects[i])
  end
  
  self:addChild(worldObjects)

  self.groundObjects = groundObjects
  self.wallObjects = wallObjects
  self.worldObjects = worldObjects
end

function Game:update(e) -- Call every frame to update game state
  local dt = e.deltaTime
  
  local player = PLAYER
  player:update(dt)

end
